@@ -3,7 +3,7 @@
  *
  * GStreamer integration library for Clutter.
  *
- * video-player.c - A simple fullscreen video player with an OSD.
+ * video-player.c - A simple video player with an OSD.
  *
  * Copyright (C) 2007,2008 OpenedHand
  *
@@ -28,8 +28,8 @@
 #include <clutter/clutter.h>
 #include <clutter-gst/clutter-gst.h>
 
-#define SEEK_H 20
-#define SEEK_W 690
+#define SEEK_H 14
+#define SEEK_W 440
 
 typedef struct _VideoApp
 {
@@ -52,6 +52,28 @@ typedef struct _VideoApp
 
 static void show_controls (VideoApp *app, gboolean vis);
 
+static gboolean opt_fullscreen = FALSE;
+static gboolean opt_loop = FALSE;
+
+static GOptionEntry options[] =
+{
+  { "fullscreen",
+    'f', 0,
+    G_OPTION_ARG_NONE,
+    &opt_fullscreen,
+    "Start the player in fullscreen",
+    NULL },
+
+  { "loop",
+    'l', 0,
+    G_OPTION_ARG_NONE,
+    &opt_loop,
+    "Start the video again once reached the EOS",
+    NULL },
+
+  { NULL }
+};
+
 static gboolean
 controls_timeout_cb (gpointer data)
 {
@@ -154,8 +176,6 @@ input_cb (ClutterStage *stage,
                                                   bev->x,
                                                   bev->y);
 
-          printf("got actor %p at pos %.2fx%.2f\n", actor, bev->x, bev->y);
-
           if (actor == app->control_pause || actor == app->control_play)
             {
               toggle_pause_state (app);
@@ -262,25 +282,28 @@ size_change (ClutterTexture *texture,
 }
 
 static void
-center_controls (VideoApp *app, ClutterActor *controls)
+position_controls (VideoApp     *app,
+                   ClutterActor *controls)
 {
-  gfloat x, y, stage_width, stage_height;
+  gfloat x, y, stage_width, stage_height, bg_width, bg_height;
 
   clutter_actor_get_size (app->stage, &stage_width, &stage_height);
+  clutter_actor_get_size (app->control, &bg_width, &bg_height);
 
-  x = (stage_width - clutter_actor_get_width (controls)) / 2;
-  y = stage_height - (stage_height / 3);
-
-  g_print ("setting x = %.2f, y = %.2f, width = %.2f\n",
-           x, y, clutter_actor_get_width (controls));
+  x = (int)((stage_width - bg_width ) / 2);
+  y = stage_height - bg_height - 28;
 
   clutter_actor_set_position (controls, x, y);
 }
 
 static void
-on_fullscreen (ClutterStage *stage, VideoApp *app)
+on_stage_allocation_changed (ClutterActor           *stage,
+                             ClutterActorBox        *box,
+                             ClutterAllocationFlags  flags,
+                             VideoApp               *app)
 {
-  center_controls (app, app->control);
+  position_controls (app, app->control);
+  show_controls (app, TRUE);
 }
 
 static void
@@ -296,6 +319,17 @@ tick (GObject      *object,
                           SEEK_H);
 }
 
+static void
+on_video_texture_eos (ClutterMedia *media,
+                      VideoApp     *app)
+{
+  if (opt_loop)
+    {
+      clutter_media_set_progress (media, 0.0);
+      clutter_media_set_playing (media, TRUE);
+    }
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -304,8 +338,21 @@ main (int argc, char *argv[])
   ClutterColor         stage_color = { 0x00, 0x00, 0x00, 0x00 };
   ClutterColor         control_color1 = { 73, 74, 77, 0xee };
   ClutterColor         control_color2 = { 0xcc, 0xcc, 0xcc, 0xff };
+  GError              *error = NULL;
 
-  clutter_gst_init (&argc, &argv);
+  clutter_gst_init_with_args (&argc,
+                              &argv,
+                              " - A simple video player",
+                              options,
+                              NULL,
+                              &error);
+
+  if (error)
+    {
+      g_print ("%s\n", error->message);
+      g_error_free (error);
+      return EXIT_FAILURE;
+    }
 
   if (argc < 2)
     {
@@ -315,7 +362,10 @@ main (int argc, char *argv[])
 
   stage = clutter_stage_get_default ();
   clutter_stage_set_color (CLUTTER_STAGE (stage), &stage_color);
-  clutter_stage_set_fullscreen (CLUTTER_STAGE (stage), TRUE);
+  clutter_actor_set_size (stage, 768, 576);
+  clutter_stage_set_minimum_size (CLUTTER_STAGE (stage), 640, 480);
+  if (opt_fullscreen)
+    clutter_stage_set_fullscreen (CLUTTER_STAGE (stage), TRUE);
 
   app = g_new0(VideoApp, 1);
   app->stage = stage;
@@ -324,9 +374,23 @@ main (int argc, char *argv[])
   if (app->vtexture == NULL)
     g_error("failed to create vtexture");
 
+  /* By default ClutterGst seeks to the nearest key frame (faster). However
+   * it has the weird effect that when you click on the progress bar, the fill
+   * goes to the key frame position that can be quite far from where you
+   * clicked. Using the ACCURATE flag tells playbin2 to seek to the actual
+   * frame */
+  clutter_gst_video_texture_set_seek_flags (
+                                    CLUTTER_GST_VIDEO_TEXTURE (app->vtexture),
+                                    CLUTTER_GST_SEEK_FLAG_ACCURATE);
+
+  g_signal_connect (app->vtexture,
+                    "eos",
+                    G_CALLBACK (on_video_texture_eos),
+                    app);
+
   g_signal_connect (stage,
-                    "fullscreen",
-                    G_CALLBACK (on_fullscreen),
+                    "allocation-changed",
+                    G_CALLBACK (on_stage_allocation_changed),
                     app);
 
   /* Handle it ourselves so can scale up for fullscreen better */
@@ -355,7 +419,7 @@ main (int argc, char *argv[])
   clutter_actor_set_opacity (app->control_seekbar, 0x99);
 
   app->control_label =
-    clutter_text_new_full ("Sans Bold 24",
+    clutter_text_new_full ("Sans Bold 14",
                            g_path_get_basename (argv[1]),
                            &control_color1);
 
@@ -373,17 +437,17 @@ main (int argc, char *argv[])
 
   clutter_actor_set_opacity (app->control, 0xee);
 
-  clutter_actor_set_position (app->control_play, 30, 30);
-  clutter_actor_set_position (app->control_pause, 30, 30);
+  clutter_actor_set_position (app->control_play, 22, 31);
+  clutter_actor_set_position (app->control_pause, 18, 31);
 
-  clutter_actor_set_size (app->control_seek1, SEEK_W+10, SEEK_H+10);
-  clutter_actor_set_position (app->control_seek1, 200, 100);
+  clutter_actor_set_size (app->control_seek1, SEEK_W+4, SEEK_H+4);
+  clutter_actor_set_position (app->control_seek1, 80, 57);
   clutter_actor_set_size (app->control_seek2, SEEK_W, SEEK_H);
-  clutter_actor_set_position (app->control_seek2, 205, 105);
+  clutter_actor_set_position (app->control_seek2, 82, 59);
   clutter_actor_set_size (app->control_seekbar, 0, SEEK_H);
-  clutter_actor_set_position (app->control_seekbar, 205, 105);
+  clutter_actor_set_position (app->control_seekbar, 82, 59);
 
-  clutter_actor_set_position (app->control_label, 200, 40);
+  clutter_actor_set_position (app->control_label, 82, 29);
 
   /* Add control UI to stage */
   clutter_container_add (CLUTTER_CONTAINER (stage),
@@ -391,11 +455,7 @@ main (int argc, char *argv[])
                          app->control,
                          NULL);
 
-  g_print ("start\n");
-
-  center_controls (app, app->control);
-
-  g_print ("stop\n");
+  position_controls (app, app->control);
 
   clutter_stage_hide_cursor (CLUTTER_STAGE (stage));
   clutter_actor_animate (app->control, CLUTTER_EASE_OUT_QUINT, 1000,
